@@ -18,6 +18,7 @@ import traceback
 import time
 from datetime import datetime, timedelta
 import re
+import os
 
 
 _scraper = FacebookScraper()
@@ -56,8 +57,8 @@ def unset_cookies():
     _scraper.session.cookies = cookiejar_from_dict({})
 
 
-def set_proxy(proxy):
-    _scraper.set_proxy(proxy)
+def set_proxy(proxy, verify=True):
+    _scraper.set_proxy(proxy, verify)
 
 
 def set_user_agent(user_agent):
@@ -84,6 +85,22 @@ def get_profile(
     return _scraper.get_profile(account, **kwargs)
 
 
+def get_reactors(
+    post_id: Union[str, int],
+    **kwargs,
+) -> Iterator[dict]:
+    """Get reactors for a given post ID
+    Args:
+        post_id(str): The post ID, as returned from get_posts
+        cookies (Union[dict, CookieJar, str]): Cookie jar to use.
+            Can also be a filename to load the cookies from a file (Netscape format).
+    """
+    _scraper.requests_kwargs['timeout'] = kwargs.pop('timeout', DEFAULT_REQUESTS_TIMEOUT)
+    cookies = kwargs.pop('cookies', None)
+    set_cookies(cookies)
+    return _scraper.get_reactors(post_id, **kwargs)
+
+
 def get_friends(
     account: str,
     **kwargs,
@@ -283,10 +300,9 @@ def get_posts_by_search(
     **kwargs,
 ) -> Iterator[Post]:
 
-    """Get posts from a Facebook page or group.
+    """Get posts by searching all of Facebook
     Args:
         word (str): The word for searching posts.
-        group (int): The group id.
         credentials (Optional[Tuple[str, str]]): Tuple of email and password to login before scraping.
         timeout (int): Timeout for requests.
         page_limit (int): How many pages of posts to go through.
@@ -387,6 +403,9 @@ def write_posts_to_csv(
     if encoding is None:
         encoding = locale.getpreferredencoding()
 
+    if os.path.isfile(filename):
+        raise FileExistsError(f"{filename} exists")
+
     if filename == "-":
         output_file = sys.stdout
     else:
@@ -478,6 +497,18 @@ def write_posts_to_csv(
     output_file.close()
 
 
+def get_groups_by_search(
+    word: str,
+    **kwargs,
+):
+    """Searches Facebook groups and yields ids for each result
+    on the first page"""
+    _scraper.requests_kwargs['timeout'] = kwargs.pop('timeout', DEFAULT_REQUESTS_TIMEOUT)
+    cookies = kwargs.pop('cookies', None)
+    set_cookies(cookies)
+    return _scraper.get_groups_by_search(word, **kwargs)
+
+
 def enable_logging(level=logging.DEBUG):
     handler = logging.StreamHandler()
     handler.setLevel(level)
